@@ -1,3 +1,4 @@
+
 """
 boson.py
 ====================================
@@ -723,13 +724,13 @@ class Boson(Core):
 
         """
         payload = None
-        payload_len = len(data) - 15
+        payload_len = len(data) - 17
 
         if payload_len > 0:
             try:
-                frame = struct.Struct(">BBIII{}sB".format(payload_len))
+                frame = struct.Struct(">BBIII{}sHB".format(payload_len))
                 res = frame.unpack(data)
-                start_marker, channel_id, sequence, function_id, return_code, payload, end_marker = res
+                start_marker, channel_id, sequence, function_id, return_code, payload, crc, end_marker = res
             except Exception as e:
                 self.logger.error(str(e))
                 self.logger.error("Failed to unpack payload")
@@ -747,7 +748,7 @@ class Boson(Core):
                 self.logger.error(str(e))
                 self.logger.error("Failed to unpack payload")
                 return None
-
+        
         if return_code == 0x0203:
             self.logger.warning("Boson response: range error")
         elif return_code == 0x017F:
@@ -776,17 +777,11 @@ class Boson(Core):
         unstuffed_payload = None
 
         if payload_len > 0:
-            unstuffed_payload = self._unstuff(payload) # data + crc
-
-            frame = struct.Struct(">{}sH".format(len(unstuffed_payload)-2)) # split data, crc
-            unstuffed_payload, crc = frame.unpack(unstuffed_payload)
+            unstuffed_payload = self._unstuff(payload)
 
         crc_bytes = self._crc(header_bytes, unstuffed_payload)
 
         if crc != crc_bytes:
-            self.logger.debug("Got crc {}, but expected {}".format(crc, crc_bytes))
-            self.logger.debug("Payload: {}".format(payload))
-            self.logger.debug("Frame: {}".format(data))
             self.logger.warning("Invalid checksum, but data may be OK")
 
         return unstuffed_payload
@@ -798,7 +793,7 @@ class Boson(Core):
         temp = bytearray()
 
         for byte in data:
-
+            
             if sys.version_info[0] < 3:
                 byte = ord(byte)
 
@@ -823,7 +818,7 @@ class Boson(Core):
         temp = bytearray()
         unstuff = False
 
-        for i, byte in enumerate(data):
+        for byte in data:
 
             if sys.version_info[0] < 3:
                 byte = ord(byte)
@@ -832,12 +827,7 @@ class Boson(Core):
                 temp.append(byte + 0xD)
                 unstuff = False
             elif byte == 0x9E:
-                if i == (len(data) - 1):
-                    continue
-                elif data[i+1] in [0x81, 0x91, 0xA1]:
-                    unstuff = True
-                else:
-                    continue
+                unstuff = True
             else:
                 temp.append(byte)
 
@@ -858,7 +848,7 @@ class Boson(Core):
 
             if time.time() - tstart > timeout:
                 break
-
+        
         return frame
 
     def _crc(self, header, payload=None):
@@ -869,7 +859,7 @@ class Boson(Core):
 
         if payload is not None:
             data += bytes(payload)
-
+        
         return binascii.crc_hqx(data, 0x1D0F)
 
     def _send_packet(self, function_id, data=None, receive_size=0):
@@ -911,7 +901,7 @@ class Boson(Core):
 
         if data is not None:
             payload += self._bitstuff(data)
-
+        
         payload += footer_bytes
 
         self.send(payload)
